/* ************************************************************************
 *
 * CookieKeeper - a Mozilla add-on
 * (c) 2013-2016 Yvon TANGUY
 *
 * ***** BEGIN LICENSE BLOCK **********************************************
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * ***** END LICENSE BLOCK **********************************************
 *
 * Preferences for the browser tab.
 * Note: The checkbox values are changed with preferences.js
 * using the prefsObserver.
 *
 * ************************************************************************
 */

"use strict";

const APP_ID_FIREFOX     = 0;
const APP_ID_SEAMONKEY   = 1;
const APP_ID_THUNDERBIRD = 2;
const APP_ID_KMELEON     = 3;

var gCookieKeeperBrowserXULPrefs = {
  _appid           : APP_ID_FIREFOX,
  _sanitize_cookies: null,

  init: function() {
    var appName = null;

    try {
      appName = APP_MOZ_CONST[Services.appinfo.ID].NAME;
    } catch (ex) {
      log.e("Unknown application type: " + Services.appinfo.name);
      // Use firefox as default, even if it can failed.
      appName = FIREFOX;
    }

    if (appName == SEAMONKEY) {
      this.initSeamonkey();
    } else if (appName == THUNDERBIRD || appName == KMELEON) {
      this.initThunderbird();
    } else {
      // Assume Firefox by default.
      this.initFirefox();
    }
  },

  initFirefox: function() {
    this._appid = APP_ID_FIREFOX;
    this._sanitize_cookies = "browser_checkbox_sanitize_on_shutdown_cookies";
    document.getElementById("seamonkey_checkbox_sanitize_on_shutdown_cookies").hidden = true;
    this.updateUIFirefox();
  },

  initSeamonkey: function() {
    this._appid = APP_ID_SEAMONKEY;
    this._sanitize_cookies = "seamonkey_checkbox_sanitize_on_shutdown_cookies";
    document.getElementById("browser_vbox_private_browsing").hidden = true;
    document.getElementById("browser_checkbox_sanitize_on_shutdown_cookies").hidden = true;
    this.updateUISeamonkey();
  },

  initThunderbird: function() {
    this._appid = APP_ID_THUNDERBIRD;
    document.getElementById("browser_vbox_private_browsing").hidden = true;
    document.getElementById("browser_vbox_sanitize").hidden = true;
    this.updateUIThunderbird();
  },

  initKMeleon: function() {
    this.initThunderbird();
    this._appid = APP_ID_KMELEON;
  },

  updateUI: function() {
    // Delay the update, wait the
    // UI items to be checked/selected.
    var that = this;
    window.setTimeout(function() {
      try {
        switch (that._appid) {
        case APP_ID_FIREFOX:
          that.updateUIFirefox();
          break;
        case APP_ID_SEAMONKEY:
          that.updateUISeamonkey();
          break;
        case APP_ID_THUNDERBIRD:
        case APP_ID_KMELEON:
          that.updateUIThunderbird();
          break;
        default:
          that.updateUIFirefox();
          break;
        }
      } catch (ex) {
        log.e(ex);
      }
    }, 0);
  },

  // Get Element
  ge: function(el) { return document.getElementById(el); },
  // Radio selected
  rs: function(el) { return this.ge(el).getAttribute("selected") == "true"; },

  updateUIFirefox: function() {
    if (this.ge("browser_checkbox_private_browsing_autostart").checked) {
      this.ge("browser_image_warn_private_browsing_autostart").hidden   = false;
      this.ge("browser_checkbox_sanitize_on_shutdown").disabled         = true;
      this.ge(this._sanitize_cookies).disabled                          = true;
      this.ge("browser_image_warn_sanitize_on_shutdown_cookies").hidden = true;
      this.ge("browser_radiogroup_accept_policy").disabled              = true;
      this.ge("browser_image_warn_accept_none").hidden                  = true;
      this.ge("browser_radiogroup_lifetime_policy").disabled            = true;
      this.ge("browser_textbox_lifetime_limit").disabled                = true;
      this.ge("browser_label_lifetime_limit").disabled                  = true;
      this.ge("browser_image_warn_lifetime_session").hidden             = true;
      this.ge("browser_checkbox_thirdparty_sessiononly").disabled       = true;

    } else {
      this.ge("browser_image_warn_private_browsing_autostart").hidden = true;
      // The rest is the same as SeaMonkey
      this.updateUISeamonkey();
    }
  },

  updateUISeamonkey: function() {

    let bcsos   = this.ge("browser_checkbox_sanitize_on_shutdown");
    let bcsosc  = this.ge(this._sanitize_cookies);
    let biwsosc = this.ge("browser_image_warn_sanitize_on_shutdown_cookies");

    bcsos.disabled  = false;
    bcsosc.disabled = !bcsos.checked;
    biwsosc.hidden  = !(bcsos.checked && bcsosc.checked);

    this.ge("browser_radiogroup_accept_policy").disabled = false;

    // The rest is the same as Thunderbird
    this.updateUIThunderbird();
  },

  updateUIThunderbird: function() {
    if (this.rs("browser_radio_accept_none")) {
      this.ge("browser_image_warn_accept_none").hidden            = false;
      this.ge("browser_radiogroup_lifetime_policy").disabled      = true;
      this.ge("browser_textbox_lifetime_limit").disabled          = true;
      this.ge("browser_label_lifetime_limit").disabled            = true;
      this.ge("browser_image_warn_lifetime_session").hidden       = true;
      this.ge("browser_checkbox_thirdparty_sessiononly").disabled = true;

    } else {
      this.ge("browser_image_warn_accept_none").hidden            = true;
      this.ge("browser_radiogroup_lifetime_policy").disabled      = false;

      if (this.rs("browser_radio_lifetime_session")) {
        this.ge("browser_image_warn_lifetime_session").hidden       = false;
        this.ge("browser_checkbox_thirdparty_sessiononly").disabled = true;
        this.ge("browser_textbox_lifetime_limit").disabled          = true;
        this.ge("browser_label_lifetime_limit").disabled            = true;

      } else {
        this.ge("browser_image_warn_lifetime_session").hidden       = true;
        this.ge("browser_checkbox_thirdparty_sessiononly").disabled = this.rs("browser_radio_accept_rtp");

        if (this.rs("browser_radio_lifetime_limit")) {
          this.ge("browser_textbox_lifetime_limit").disabled = false;
          this.ge("browser_label_lifetime_limit").disabled   = false;

        } else {
          this.ge("browser_textbox_lifetime_limit").disabled = true;
          this.ge("browser_label_lifetime_limit").disabled   = true;
        }
      }
    }
  }
};

var gCookieKeeperBrowserHTMLPrefs = {
  init: function() {
    this.uninit();

    var appName = null;
    try {
      appName = APP_MOZ_CONST[Services.appinfo.ID].NAME;
    } catch (ex) {
      log.e("Unknown application type: " + Services.appinfo.name);
      // Use firefox as default, even if it can failed.
      appName = FIREFOX;
    }

    var cbPrivBrowsingAuto   = document.getElementById("browser_cb_PrivateBrowsingAutoStart");
    var cbSanitizeOnShutdown = document.getElementById("browser_cb_SanitizeOnShutdown");
    var formAcceptPolicy     = document.getElementById("browser_form_accept_policy");
    var formLifetimePolicy   = document.getElementById("browser_form_lifetime_policy");
    var hLifetimeDays        = document.getElementById("browser_input_lifetime_policy_days");

    cbPrivBrowsingAuto.addEventListener(EVENT_CUSTOM_CHANGED,   this.onPermChange, false);
    cbSanitizeOnShutdown.addEventListener(EVENT_CUSTOM_CHANGED, this.onPermChange, false);
    formAcceptPolicy.addEventListener(EVENT_CUSTOM_CHANGED,     this.onPermChange, false);
    formLifetimePolicy.addEventListener(EVENT_CUSTOM_CHANGED,   this.onPermChange, false);
    hLifetimeDays.addEventListener(EVENT_CUSTOM_CHANGED,        this.onLifetimeChanged, false);

    if (appName == FIREFOX || appName == PALEMOON) {
      this.initFirefox();
    } else if (appName == SEAMONKEY) {
      this.initSeamonkey();
    } else if (appName == THUNDERBIRD || appName == KMELEON) {
      this.initThunderbird();
    } else {
      this.initFirefox();
    }

    this.onPermChange();
    this.onLifetimeChanged();
  },

  initFirefox: function() {
    let hDiv = document.getElementById("browser_div_ClearOnShutDownCookies_seamonkey");
    hDiv.parentElement.removeChild(hDiv);
  },

  initSeamonkey: function() {
    let hDiv = document.getElementById("browser_PrivateBrowsingAutoStart");
    hDiv.style.display = "none";
    hDiv = document.getElementById("browser_div_ClearOnShutDownCookies_all");
    hDiv.parentElement.removeChild(hDiv);
  },

  initThunderbird: function() {
    let hDiv = document.getElementById("browser_PrivateBrowsingAutoStart");
    hDiv.style.display = "none";
    hDiv = document.getElementById("browser_SanitizeOnShutdown");
    hDiv.style.display = "none";
  },

  uninit: function() {
    let cbPrivBrowsingAuto   = document.getElementById("browser_cb_PrivateBrowsingAutoStart");
    let cbSanitizeOnShutdown = document.getElementById("browser_cb_SanitizeOnShutdown");
    let formAcceptPolicy     = document.getElementById("browser_form_accept_policy");
    let formLifetimePolicy   = document.getElementById("browser_form_lifetime_policy");
    let hLifetimeDays        = document.getElementById("browser_input_lifetime_policy_days");

    cbPrivBrowsingAuto.removeEventListener(EVENT_CUSTOM_CHANGED,   this.onPermChange, false);
    cbSanitizeOnShutdown.removeEventListener(EVENT_CUSTOM_CHANGED, this.onPermChange, false);
    formAcceptPolicy.removeEventListener(EVENT_CUSTOM_CHANGED,     this.onPermChange, false);
    formLifetimePolicy.removeEventListener(EVENT_CUSTOM_CHANGED,   this.onPermChange, false);
    hLifetimeDays.removeEventListener(EVENT_CUSTOM_CHANGED,        this.onLifetimeChanged, false);
  },

  /**
   * @brief Update enbaled/disabled items, depending of the values.
   * @par
   * For example, if we always are in private browsing, all others options are meaningless
   * so disabled them.
   */
  onPermChange: function(evtChange) {
    let that                 = gCookieKeeperBrowserHTMLPrefs;
    let cbPrivBrowsingAuto   = document.getElementById("browser_cb_PrivateBrowsingAutoStart");
    let acceptPolicyValue    = document.getElementById("browser_form_accept_policy").value;
    let lifetimePolicyValue  = document.getElementById("browser_form_lifetime_policy").value;

    if (cbPrivBrowsingAuto.checked) {
      that.disableCheckBox("browser_cb_SanitizeOnShutdown",     true);
      that.disableCheckBox("browser_cb_ClearOnShutDownCookies", true);
      that.disableRadioGroup("browser_form_accept_policy",      true);
      that.disableRadioGroup("browser_form_lifetime_policy",    true);
      that.disableCheckBox("browser_cb_lifetime_policy_tpso",   true);
      document.getElementById("browser_input_lifetime_policy_days").setAttribute("disabled", "disabled");
      return;
    }

    that.disableCheckBox("browser_cb_SanitizeOnShutdown", false);
    if (document.getElementById("browser_cb_SanitizeOnShutdown").checked) {
      that.disableCheckBox("browser_cb_ClearOnShutDownCookies", false);
    } else {
      that.disableCheckBox("browser_cb_ClearOnShutDownCookies", true);
    }

    that.disableRadioGroup("browser_form_accept_policy", false);

    if (acceptPolicyValue == "2") {
      // "Reject all cookies" is selected, disable all lifetime policy elements
      that.disableRadioGroup("browser_form_lifetime_policy",  true);
      that.disableCheckBox("browser_cb_lifetime_policy_tpso", true);
      document.getElementById("browser_input_lifetime_policy_days").setAttribute("disabled", "disabled");
      return;
    }

    // Enable lifetime policy elements
    that.disableRadioGroup("browser_form_lifetime_policy",  false);
    that.disableCheckBox("browser_cb_lifetime_policy_tpso", acceptPolicyValue == "1");

    if (lifetimePolicyValue == "3") {
      document.getElementById("browser_input_lifetime_policy_days").removeAttribute("disabled");
    } else {
      document.getElementById("browser_input_lifetime_policy_days").setAttribute("disabled", "disabled");
    }
  },

  onLifetimeChanged: function(evtChanged) {
    let value = document.getElementById("browser_input_lifetime_policy_days").value;
    let hElem = document.getElementById("browser_label_lifetime_policy_days");
    if (hElem.hasAttribute("data-l10n-dynid")) {
      let l10n   = hElem.getAttribute("data-l10n-dynid");
      let trText = gChrome.i18n.getMessage(l10n, [value]);
      hElem.textContent = trText;
    }
  },

  disableRadioGroup: function(name, disabled) {
    // Dirty way to disable radio input.
    // HTML is not very convenient for making UI :(
    let form = document.getElementById(name);
    if (form != null) {
      let nRadio = form.children.length;
      for (let i=0; i<nRadio; i++) {
        let hChild = form.children[i];
        if (disabled) {
          if (hChild.nodeName == "INPUT") {
            hChild.setAttribute("disabled", "disabled");
          } else if (hChild.nodeName == "LABEL") {
            hChild.classList.add("label_disabled");
          }
        } else {
          // enable the radio
          if (hChild.nodeName == "INPUT") {
            hChild.removeAttribute("disabled");
          } else if (hChild.nodeName == "LABEL") {
            hChild.classList.remove("label_disabled");
          }
        }
      }
    }
  },

  disableCheckBox: function(name, disabled) {
    let hInput = document.getElementById(name);
    if (hInput != null) {
      if (disabled) {
        hInput.setAttribute("disabled", "disabled");
        hInput.nextElementSibling.classList.add("label_disabled");
      } else {
        hInput.removeAttribute("disabled");
        hInput.nextElementSibling.classList.remove("label_disabled");
      }
    }
  }
};
